#!/usr/bin/env python3
"""
Unit tester for core.py

Dependencies:
 - python3.6+
   - unittest
   - pandas >= 0.22.0
"""

import unittest
import pandas as pd
from primer_tk import core

class TestCoreSV(unittest.TestCase):
    """
    Test all core functions for SV pipeline.
    """
    def setUp(self):
        """
        Inputs for each function.
        """
        pass
    def tearDown(self):
        pass

if __name__ == "__main__":
    unittest.main()
